@@ -43,6 +43,8 @@ void zpool_no_memory(void);
 uint_t num_logs(nvlist_t *nv);
 uint64_t array64_max(uint64_t array[], unsigned int len);
 int isnumber(char *str);
+int highbit64(uint64_t i);
+int lowbit64(uint64_t i);
 
 /*
  * Misc utility functions
