@@ -46,7 +46,7 @@ struct znode;
 
 /*
  * This structure emulates the vfs_t from other platforms.  It's purpose
- * is to faciliate the handling of mount options and minimize structural
+ * is to facilitate the handling of mount options and minimize structural
  * differences between the platforms.
  */
 typedef struct vfs {
@@ -105,7 +105,7 @@ struct zfsvfs {
 	list_t		z_all_znodes;	/* all znodes in the fs */
 	uint64_t	z_nr_znodes;	/* number of znodes in the fs */
 	unsigned long	z_rollback_time; /* last online rollback time */
-	unsigned long	z_snap_defer_time; /* last snapshot unmount deferal */
+	unsigned long	z_snap_defer_time; /* last snapshot unmount deferral */
 	kmutex_t	z_znodes_lock;	/* lock for z_all_znodes */
 	arc_prune_t	*z_arc_prune;	/* called by ARC to prune caches */
 	struct inode	*z_ctldir;	/* .zfs directory inode */
