@@ -635,7 +635,7 @@ crypto_mac_single(crypto_context_t context, crypto_data_t *data,
 	return (error);
 }
 
-#if defined(_KERNEL) && defined(HAVE_SPL)
+#if defined(_KERNEL)
 EXPORT_SYMBOL(crypto_mac_prov);
 EXPORT_SYMBOL(crypto_mac);
 EXPORT_SYMBOL(crypto_mac_verify_prov);
