@@ -152,17 +152,15 @@ dmu_zfetch_fini(zfetch_t *zf)
 static void
 dmu_zfetch_stream_create(zfetch_t *zf, uint64_t blkid)
 {
-	zstream_t *zs;
 	zstream_t *zs_next;
 	int numstreams = 0;
-	uint32_t max_streams;
 
 	ASSERT(RW_WRITE_HELD(&zf->zf_rwlock));
 
 	/*
 	 * Clean up old streams.
 	 */
-	for (zs = list_head(&zf->zf_stream);
+	for (zstream_t *zs = list_head(&zf->zf_stream);
 	    zs != NULL; zs = zs_next) {
 		zs_next = list_next(&zf->zf_stream, zs);
 		if (((gethrtime() - zs->zs_atime) / NANOSEC) >
@@ -180,7 +178,7 @@ dmu_zfetch_stream_create(zfetch_t *zf, uint64_t blkid)
 	 * If we are already at the maximum number of streams for this file,
 	 * even after removing old streams, then don't create this stream.
 	 */
-	max_streams = MAX(1, MIN(zfetch_max_streams,
+	uint32_t max_streams = MAX(1, MIN(zfetch_max_streams,
 	    zf->zf_dnode->dn_maxblkid * zf->zf_dnode->dn_datablksz /
 	    zfetch_max_distance));
 	if (numstreams >= max_streams) {
@@ -188,7 +186,7 @@ dmu_zfetch_stream_create(zfetch_t *zf, uint64_t blkid)
 		return;
 	}
 
-	zs = kmem_zalloc(sizeof (*zs), KM_SLEEP);
+	zstream_t *zs = kmem_zalloc(sizeof (*zs), KM_SLEEP);
 	zs->zs_blkid = blkid;
 	zs->zs_pf_blkid = blkid;
 	zs->zs_ipf_blkid = blkid;
@@ -211,13 +209,24 @@ dmu_zfetch(zfetch_t *zf, uint64_t blkid, uint64_t nblks, boolean_t fetch_data)
 {
 	zstream_t *zs;
 	int64_t pf_start, ipf_start, ipf_istart, ipf_iend;
-	int64_t pf_ahead_blks, max_blks, iblk;
-	int epbs, max_dist_blks, pf_nblks, ipf_nblks, i;
+	int64_t pf_ahead_blks, max_blks;
+	int epbs, max_dist_blks, pf_nblks, ipf_nblks;
 	uint64_t end_of_access_blkid;
 	end_of_access_blkid = blkid + nblks;
+	spa_t *spa = zf->zf_dnode->dn_objset->os_spa;
+	krw_t rw = RW_READER;
 
 	if (zfs_prefetch_disable)
 		return;
+	/*
+	 * If we haven't yet loaded the indirect vdevs' mappings, we
+	 * can only read from blocks that we carefully ensure are on
+	 * concrete vdevs (or previously-loaded indirect vdevs).  So we
+	 * can't allow the predictive prefetcher to attempt reads of other
+	 * blocks (e.g. of the MOS's dnode object).
+	 */
+	if (!spa_indirect_vdevs_loaded(spa))
+		return;
 
 	/*
 	 * As a fast path for small (single-block) files, ignore access
@@ -226,7 +235,8 @@ dmu_zfetch(zfetch_t *zf, uint64_t blkid, uint64_t nblks, boolean_t fetch_data)
 	if (blkid == 0)
 		return;
 
-	rw_enter(&zf->zf_rwlock, RW_READER);
+retry:
+	rw_enter(&zf->zf_rwlock, rw);
 
 	/*
 	 * Find matching prefetch stream.  Depending on whether the accesses
@@ -264,8 +274,13 @@ dmu_zfetch(zfetch_t *zf, uint64_t blkid, uint64_t nblks, boolean_t fetch_data)
 		 * a new stream for it.
 		 */
 		ZFETCHSTAT_BUMP(zfetchstat_misses);
-		if (rw_tryupgrade(&zf->zf_rwlock))
-			dmu_zfetch_stream_create(zf, end_of_access_blkid);
+		if (rw == RW_READER && !rw_tryupgrade(&zf->zf_rwlock)) {
+			rw_exit(&zf->zf_rwlock);
+			rw = RW_WRITER;
+			goto retry;
+		}
+
+		dmu_zfetch_stream_create(zf, end_of_access_blkid);
 		rw_exit(&zf->zf_rwlock);
 		return;
 	}
@@ -338,18 +353,18 @@ dmu_zfetch(zfetch_t *zf, uint64_t blkid, uint64_t nblks, boolean_t fetch_data)
 	 * calling it to reduce the time we hold them.
 	 */
 
-	for (i = 0; i < pf_nblks; i++) {
+	for (int i = 0; i < pf_nblks; i++) {
 		dbuf_prefetch(zf->zf_dnode, 0, pf_start + i,
 		    ZIO_PRIORITY_ASYNC_READ, ARC_FLAG_PREDICTIVE_PREFETCH);
 	}
-	for (iblk = ipf_istart; iblk < ipf_iend; iblk++) {
+	for (int64_t iblk = ipf_istart; iblk < ipf_iend; iblk++) {
 		dbuf_prefetch(zf->zf_dnode, 1, iblk,
 		    ZIO_PRIORITY_ASYNC_READ, ARC_FLAG_PREDICTIVE_PREFETCH);
 	}
 	ZFETCHSTAT_BUMP(zfetchstat_hits);
 }
 
-#if defined(_KERNEL) && defined(HAVE_SPL)
+#if defined(_KERNEL)
 /* BEGIN CSTYLED */
 module_param(zfs_prefetch_disable, int, 0644);
 MODULE_PARM_DESC(zfs_prefetch_disable, "Disable all ZFS prefetching");
