@@ -768,7 +768,7 @@ dsl_delegation_on(objset_t *os)
 	return (!!spa_delegation(os->os_spa));
 }
 
-#if defined(_KERNEL) && defined(HAVE_SPL)
+#if defined(_KERNEL)
 EXPORT_SYMBOL(dsl_deleg_get);
 EXPORT_SYMBOL(dsl_deleg_set);
 #endif
