@@ -82,8 +82,10 @@ zrl_add_impl(zrlock_t *zrl, const char *zc)
 				ASSERT3S((int32_t)n, >=, 0);
 #ifdef	ZFS_DEBUG
 				if (zrl->zr_owner == curthread) {
-					DTRACE_PROBE2(zrlock__reentry,
-					    zrlock_t *, zrl, uint32_t, n);
+					DTRACE_PROBE3(zrlock__reentry,
+					    zrlock_t *, zrl,
+					    kthread_t *, curthread,
+					    uint32_t, n);
 				}
 				zrl->zr_owner = curthread;
 				zrl->zr_caller = zc;
@@ -157,11 +159,9 @@ zrl_exit(zrlock_t *zrl)
 int
 zrl_refcount(zrlock_t *zrl)
 {
-	int n;
-
 	ASSERT3S(zrl->zr_refcount, >, ZRL_DESTROYED);
 
-	n = (int)zrl->zr_refcount;
+	int n = (int)zrl->zr_refcount;
 	return (n <= 0 ? 0 : n);
 }
 
@@ -189,7 +189,7 @@ zrl_owner(zrlock_t *zrl)
 }
 #endif
 
-#if defined(_KERNEL) && defined(HAVE_SPL)
+#if defined(_KERNEL)
 
 EXPORT_SYMBOL(zrl_add_impl);
 EXPORT_SYMBOL(zrl_remove);
